require net
require recode
require tel
require reverse_config
## requires /usr/lib/callmonitor/reverse/*.sh

## resolve numbers to names and addresses; returns 1 if no lookup
## performed or if there were errors (no need to cache)
##
## The resulting name and address should be returned in Latin-1 encoding
##
## The provider may be chosen automatically ($2 is empty) or by hand

reverse_lookup() {
    local number_plain=$1 prov=$2
    local area_prov= child afile name exit number __
    normalize_address "$number_plain" || return 1
    number=$__
    empty "$number" && return 1

    if [ -z "$prov" ]; then
	_reverse_choose_provider "$number"
    else
	_reverse_require_provider "$prov"
    fi

    if [ -n "$area_prov" ]; then
	afile="/var/run/phonebook/lookup-$area_prov-$number"
	_reverse_lookup "$area_prov" "$number" | _reverse_atomic "$afile" & child=$!
    fi
    name=$(_reverse_lookup "$prov" "$number"); exit=$?
    if ! empty "$name"; then
	echo "$name"
	exit=0
    elif [ -n "$area_prov" ]; then
	## wait for area provider to finish
	wait $child
	name=$(cat "$afile" 2>/dev/null); exit=$?
	if ! empty "$name"; then
	    ## $name is only country/city
	    echo "$(normalize_address "$number" display; echo $__); $name"
	    exit=0
	fi
    fi
    if [ -n "$area_prov" ]; then
	{ kill "$child" 2>/dev/null; rm -f "$afile"; } &
    fi
    return $exit
}
_reverse_choose_provider() {
    local number=$1
    local lkz=$(tel_lkz "$number")
    if [ -z "$lkz" ]; then
	lkz=other
    fi
    REVERSE_PROVIDER_get "$lkz" prov
    area_prov=$AREA_PROVIDER
}
_reverse_atomic() {
    local file=$1 tmp=$1.tmp
    cat > "$tmp" && mv "$tmp" "$file" || rm "$tmp"
}

_reverse_lookup() {
    local exit=0 result
    result=$(_reverse_query_provider "$@"); exit=$?
    case $result in
	OK:*) echo "${result#OK:}"; return 0 ;;
	NA:*) echo ""; return 1 ;;
	*)    echo ""; return 2 ;;
    esac
}

_reverse_query_provider() {
    local prov=$1 number=$2 exit=0
    if empty "$prov"; then return 0; fi
    eval $({
	{ _reverse_${prov}_request "$number"; echo exit=$? >&4; } |
	_reverse_${prov}_extract
    } 4>&1 >&9)
## 141: nc: Broken pipe
    return $(( exit == 141 ? 0 : exit ))
} 9>&1

_reverse_lookup_url() {
    local prov=$1 number=$2 URL=
    _reverse_${prov}_url "$number"
    echo "$URL"
}

## sed snippets used by provider plugins

readonly REVERSE_OK='s/^/OK:/; p; q'
readonly REVERSE_NA='s/.*/NA:/; p; q'

readonly REVERSE_DECODE_ENTITIES='
    s#&uuml;##g
    s#&auml;##g
    s#&ouml;##g
    s#&Uuml;##g
    s#&Auml;##g
    s#&Ouml;##g
    s#&szlig;##g
'
readonly REVERSE_DECODE_ENTITIES_UTF8='
    s#&uuml;#'$'\xc3\xbc''#g
    s#&auml;#'$'\xc3\xa4''#g
    s#&ouml;#'$'\xc3\xb6''#g
    s#&Uuml;#'$'\xc3\x9c''#g
    s#&Auml;#'$'\xc3\x84''#g
    s#&Ouml;#'$'\xc3\x96''#g
    s#&szlig;#'$'\xc3\x9f''#g
'
readonly REVERSE_SANITIZE='
    s#</rev:name>#\&PART;#g
    s#<[^>]*># #g
    s#&nbsp;# #g
    s#&lt;#<#g
    s#&gt;#>#g
    s#&quot;#"#g
    s#&apos;#'\''#g
    s#|#,#g
    s#&PART;#|#g
    s#&\([^a]\|a[^m]\|am[^p]\|amp[A-Za-z]\)[A-Za-z]*;##g
    s#&amp;#\&#g
    s#;#,#g
    s#|#;#g
    s#\([[:space:]]\| \)\+# #g
    s#,\( *,\)\+#,#g
    s#; *,#;#g
    s#, *;#;#g
    s#^[,; ]*##
    s# \([],;)]\)#\1#g
    s#\([([]\) #\1#g
    s#[,; ]*$##
'

## initialization: load required provider modules

_reverse_require_provider() {
    local prov=$1
    if ! type "_reverse_${prov}_request" >/dev/null; then
	_reverse_load "$prov"
    fi
}
_reverse_load() {
    local provider=$1
    if [ -z "$provider" ]; then return; fi
    local file=$CALLMONITOR_LIBDIR/reverse/$provider.sh
    if [ -e "$file" ]; then
	. "$file"
    fi
    if [ -n "$REVERSE_DEV" -a -e "/mod/root/$provider.sh" ]; then
	. "$file"
    fi
}
_reverse_init() {
    local entry prov
    for entry in $REVERSE_PROVIDER $AREA_PROVIDER; do
	prov=${entry#*:}
	_reverse_load "$prov"
    done
    unset -f _reverse_init
}
_reverse_init
